.swh-org dl dd{margin-left:9px}.swh-org img,.swh-org table{margin-bottom:1rem}.swh-org table,.swh-org table td,.swh-org table th{padding:6px 13px;border:1px solid #dfe2e5}.swh-org table tr:nth-child(2n){background-color:#f2f2f2}.swh-org span.task-status{padding:0 .2em;margin-right:10px;color:#fff}.swh-org span.task-status.todo{color:#fff;background-color:#2795b6}.swh-org span.task-status.done{color:#fff;background-color:#518f1f}.swh-org .section-number{padding-right:5px;font-weight:700}.swh-org .org-subscript-child{font-size:85%}
/*# sourceMappingURL=org.67476f903a00cae36e53.css.map */