/**
 * Copyright (C) 2019  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.octicon-git-commit::before {
    display: inline-block;
    width: 14px;
    height: 14px;
    vertical-align: middle;
    text-align: center;
    content: '';
    mask: url('~octicons/build/svg/git-commit.svg') no-repeat 50% 50%;
    mask-size: cover;
    background-color: currentColor;
    margin-left: 2px;
    margin-right: 2px;
}
