import module1, module2, module3
from module4 import nil

type
  TFoo = object ## Doc comment
    a: int32
  PFoo = ref TFoo

proc do_stuff314(param_1: TFoo, par2am: var PFoo): PFoo {.exportc: "dostuff" .} =
  # Regular comment
  discard """
  dfag
sdfg""
"""
  result = nil

method abc(a: TFoo) = discard 1u32 + 0xabcdefABCDEFi32 + 0o01234567i8 + 0b010

discard rawstring"asdf""adfa"
var normalstring = "asdf"
let a: uint32 = 0xFFaF'u32
